@@ -83,7 +83,8 @@ static inline uint32_t mul_mod_barrett(const uint32_t x, const uint32_t y,
 static inline void non_avx_matrix_vector_product(uint32_t* vec_res, const uint32_t* mat,
                                          const uint32_t* vec, const uint32_t ncols,
                                          const uint32_t nrows, const uint32_t PRIME,
-                                         const uint32_t RED_32, const uint32_t RED_64)
+					 const uint32_t RED_32, const uint32_t RED_64,
+					 stat_t *st)
 {
     uint32_t i, j;
     int64_t prod1, prod2, prod3, prod4;
@@ -177,7 +178,7 @@ static inline void non_avx_matrix_vector_product(uint32_t* vec_res, const uint32
                 prod4 +=  ((prod4 >> 63)) & modsquare;
                 i     +=  1;
             }
-            /* enusre prod being positive */
+            /* ensure prod being positive */
             prod1 =   -prod1;
             prod1 +=  (prod1 >> 63) & modsquare;
             prod2 =   -prod2;
@@ -222,7 +223,7 @@ static inline void non_avx_matrix_vector_product(uint32_t* vec_res, const uint32
             prod1 +=  ((prod1 >> 63)) & modsquare;
             i     +=  1;
         }
-        /* enusre prod being positive */
+        /* ensure prod being positive */
         prod1 =   -prod1;
         prod1 +=  (prod1 >> 63) & modsquare;
         vec_res[j]    = (uint32_t)(prod1 % PRIME);
@@ -623,7 +624,6 @@ static inline void _4mul_new_matrix_vector_product(uint32_t* vec_res, const uint
 AVX2-based matrix vector product
 **/
 
-
 static inline void _8mul_matrix_vector_product(uint32_t* vec_res,
                                                const uint32_t* mat,
                                                const uint32_t* vec,
@@ -633,51 +633,57 @@ static inline void _8mul_matrix_vector_product(uint32_t* vec_res,
                                                const uint32_t PRIME,
                                                const uint32_t RED_32,
                                                const uint32_t RED_64,
-                                               const uint32_t preinv){
-  __m256i acc_low,acc_high,mask,vec8,mat8;
-  __m256i prod1,prod2;
-  __m256i res1;
-#if IS_ALIGNED_TMP
-    uint64_t acc4x64[8] ALIGNED32;
-#else
-    uint64_t acc4x64[8];
-#endif
-    uint64_t acc64;
-    const uint32_t *vec_cp;
-
-    unsigned int i,j;
-
+                                               const uint32_t preinv,
+					       stat_t *st){
     //mask pour recuperer les parties basses
-    mask=AVX2SET1_64(MONE32);
+    __m256i mask=AVX2SET1_64(MONE32);
     const long quo0 = LENGTHQ8(ncols);
     const long rem0 = LENGTHR8(ncols);
 
-    /* For each row of the matrix, we compute a dot product */
-    for(j = 0; j < nrows; ++j){
-      vec_cp=vec;
-
-      acc_low=AVX2SETZERO();
-      acc_high=AVX2SETZERO();
-
-      long local_ncols, quo, rem;
-      if(dst[j] != 0){
-        local_ncols = ncols - dst[j];
-        quo = LENGTHQ8(local_ncols);
-        rem = LENGTHR8(local_ncols);
-      }
-      else{
-        local_ncols = ncols;
-        quo = quo0;
-        rem = rem0;
-      }
-      const long bs =  quo;
-      /* Accumulation */
-      for(i = 0 ; i < bs; ++i){
-
-#if IS_ALIGNED_MATRIX
-          mat8=AVX2LOAD(mat);
+#pragma omp parallel num_threads (st->nthrds)
+    {
+      unsigned int i,j;
+#if IS_ALIGNED_TMP
+      uint64_t acc4x64[8] ALIGNED32;
 #else
-          mat8=AVX2LOADU(mat);
+      uint64_t acc4x64[8];
+#endif
+      uint64_t acc64;
+      __m256i acc_low,acc_high,vec8,mat8;
+      __m256i prod1,prod2;
+      __m256i res1;
+      const uint32_t *vec_cp;
+      const uint32_t *mat_cp;
+    
+      /* parallelization of the outer loop (rows) */
+#pragma omp for
+      /* For each row of the matrix, we compute a dot product */
+      for(j = 0; j < nrows; ++j){
+	vec_cp=vec;
+	mat_cp=mat + j*ncols;
+	
+	acc_low=AVX2SETZERO();
+	acc_high=AVX2SETZERO();
+	
+	long local_ncols, quo, rem;
+	if(dst[j] != 0){
+	  local_ncols = ncols - dst[j];
+	  quo = LENGTHQ8(local_ncols);
+	  rem = LENGTHR8(local_ncols);
+	}
+	else{
+	  local_ncols = ncols;
+	  quo = quo0;
+	  rem = rem0;
+	}
+	const long bs =  quo;
+	/* Accumulation */
+	for(i = 0 ; i < bs; ++i){
+	  
+#if IS_ALIGNED_MATRIX
+          mat8=AVX2LOAD(mat_cp);
+#else
+          mat8=AVX2LOADU(mat_cp);
 #endif
 #if IS_ALIGNED_VECTOR
           vec8=AVX2LOAD(vec_cp);
@@ -689,12 +695,12 @@ static inline void _8mul_matrix_vector_product(uint32_t* vec_res,
           prod1=AVX2MUL(mat8,vec8);
           prod2= AVX2MUL(AVX2SRLI_64(mat8,32),AVX2SRLI_64(vec8,32));
           res1 = prod1 + prod2;
-          mat+=8;
+          mat_cp+=8;
           vec_cp+=8;
 #if IS_ALIGNED_MATRIX
-          mat8=AVX2LOAD(mat);
+          mat8=AVX2LOAD(mat_cp);
 #else
-          mat8=AVX2LOADU(mat);
+          mat8=AVX2LOADU(mat_cp);
 #endif
 #if IS_ALIGNED_VECTOR
           vec8=AVX2LOAD(vec_cp);
@@ -704,12 +710,12 @@ static inline void _8mul_matrix_vector_product(uint32_t* vec_res,
           prod1=AVX2MUL(mat8,vec8);
           prod2= AVX2MUL(AVX2SRLI_64(mat8,32),AVX2SRLI_64(vec8,32));
           res1 += prod1 + prod2;
-          mat+=8;
+          mat_cp+=8;
           vec_cp+=8;
 #if IS_ALIGNED_MATRIX
-          mat8=AVX2LOAD(mat);
+          mat8=AVX2LOAD(mat_cp);
 #else
-          mat8=AVX2LOADU(mat);
+          mat8=AVX2LOADU(mat_cp);
 #endif
 #if IS_ALIGNED_VECTOR
           vec8=AVX2LOAD(vec_cp);
@@ -719,12 +725,12 @@ static inline void _8mul_matrix_vector_product(uint32_t* vec_res,
           prod1=AVX2MUL(mat8,vec8);
           prod2= AVX2MUL(AVX2SRLI_64(mat8,32),AVX2SRLI_64(vec8,32));
           res1 += prod1 + prod2;
-          mat+=8;
+          mat_cp+=8;
           vec_cp+=8;
 #if IS_ALIGNED_MATRIX
-          mat8=AVX2LOAD(mat);
+          mat8=AVX2LOAD(mat_cp);
 #else
-          mat8=AVX2LOADU(mat);
+          mat8=AVX2LOADU(mat_cp);
 #endif
 #if IS_ALIGNED_VECTOR
           vec8=AVX2LOAD(vec_cp);
@@ -734,43 +740,41 @@ static inline void _8mul_matrix_vector_product(uint32_t* vec_res,
           prod1=AVX2MUL(mat8,vec8);
           prod2= AVX2MUL(AVX2SRLI_64(mat8,32),AVX2SRLI_64(vec8,32));
           res1 += prod1 + prod2;
-          mat+=8;
+          mat_cp+=8;
           vec_cp+=8;
 
           acc_low=AVX2ADD_64(acc_low,AVX2AND_(res1,mask));
           acc_high=AVX2ADD_64(acc_high,AVX2SRLI_64(res1,32));
 
-      }
-
+	}
+	
 #if IS_ALIGNED_TMP
-      AVX2STORE(acc4x64,acc_low);
-      AVX2STORE(acc4x64+4,acc_high);
+	AVX2STORE(acc4x64,acc_low);
+	AVX2STORE(acc4x64+4,acc_high);
 #else
-      AVX2STOREU(acc4x64,acc_low);
-      AVX2STOREU(acc4x64+4,acc_high);
+	AVX2STOREU(acc4x64,acc_low);
+	AVX2STOREU(acc4x64+4,acc_high);
 #endif
-
-      /* Reduction */
-      acc64=0;
-      for(i=0;i<4;++i){
-        //partie haute du registre haut (2^64 ->2^95)
-        acc4x64[i]+=((acc4x64[i+4]>>32)*RED_64)%PRIME;
-        //partie basse du registre haut (2^32->2^63)
-        acc4x64[i]+=((acc4x64[i+4]&((uint64_t)0xFFFFFFFF))*RED_32)%PRIME;
-        acc64+=acc4x64[i]%PRIME;
-        //acc64+=acc4x64[i];
+	
+	/* Reduction */
+	acc64=0;
+	for(i=0;i<4;++i){
+	  //partie haute du registre haut (2^64 ->2^95)
+	  acc4x64[i]+=((acc4x64[i+4]>>32)*RED_64)%PRIME;
+	  //partie basse du registre haut (2^32->2^63)
+	  acc4x64[i]+=((acc4x64[i+4]&((uint64_t)0xFFFFFFFF))*RED_32)%PRIME;
+	  acc64+=acc4x64[i]%PRIME;
+	}
+	
+	/* *vec_res[j]=MODRED32(acc64, PRIME, preinv); */
+	vec_res[j]=acc64%PRIME;
+	
+	long tmp = 0;
+	for(long k = 0; k < rem; k++){
+	  tmp += ((long)mat_cp[k] * (long)vec_cp[k]) % PRIME;
+	}
+	vec_res[j] += tmp % PRIME;
       }
-
-      /* *vec_res=MODRED32(acc64, PRIME, preinv); */
-      *vec_res=acc64%PRIME; 
-
-      long tmp = 0;
-      for(long k = 0; k < rem; k++){
-        tmp += ((long)mat[k] * (long)vec_cp[k]) % PRIME;
-      }
-      *vec_res += tmp % PRIME;
-      mat+=(rem + dst[j]);
-      ++vec_res;
     }
 }
 #endif
