@@ -86,16 +86,26 @@ static inline void store_exponent(const char *term, data_gens_ff_t *gens, int32_
 
 static inline void display_monomial(FILE *file, data_gens_ff_t *gens, int64_t pos,
                                     int32_t **bexp){
-  int32_t exp;
-  for(int k = 0; k < gens->nvars; k++){
-    exp = (*bexp)[(pos)*gens->nvars + k];
+  int32_t exp = 0;
+  for(int k = 0; k < gens->nvars - gens->elim; k++){
+    exp = (*bexp)[(pos)*(gens->nvars - gens->elim) + k];
+    if(exp > 0){
+      break;
+    }
+  }
+  if(exp == 0){
+    fprintf(file, "1");
+    return;
+  }
+  for(int k = 0; k < (gens->nvars - gens->elim); k++){
+    exp = (*bexp)[(pos)*(gens->nvars - gens->elim) + k];
     if(exp > 0){
       fprintf(file, "*");
       if(exp==1){
-        fprintf(file, "%s", gens->vnames[k]);
+        fprintf(file, "%s", gens->vnames[k + gens->elim]);
       }
       else{
-        fprintf(file, "%s^%d",gens->vnames[k], exp);
+        fprintf(file, "%s^%d",gens->vnames[k + gens->elim], exp);
       }
     }
   }
@@ -104,23 +114,23 @@ static inline void display_monomial(FILE *file, data_gens_ff_t *gens, int64_t po
 static inline int32_t display_monomial_single(FILE *file, data_gens_ff_t *gens,
                                               int64_t pos, int32_t **bexp){
   int32_t exp, b = 0;
-  for(int k = 0; k < gens->nvars; k++){
-    exp = (*bexp)[(pos)*(gens->nvars) + k];
+  for(int k = 0; k < gens->nvars - gens->elim; k++){
+    exp = (*bexp)[(pos)*(gens->nvars - gens->elim) + k];
     if(exp > 0){
       if(exp==1){
         if(b){
-          fprintf(file, "*%s", gens->vnames[k]);
+          fprintf(file, "*%s", gens->vnames[k + gens->elim]);
         }
         else{
-          fprintf(file, "%s", gens->vnames[k]);
+          fprintf(file, "%s", gens->vnames[k + gens->elim]);
         }
       }
       else{
         if(b){
-          fprintf(file, "*%s^%d",gens->vnames[k], exp);
+          fprintf(file, "*%s^%d",gens->vnames[k + gens->elim], exp);
         }
         else{
-          fprintf(file, "%s^%d",gens->vnames[k], exp);
+          fprintf(file, "%s^%d",gens->vnames[k + gens->elim], exp);
         }
       }
       b = 1;
@@ -1027,10 +1037,11 @@ static inline void get_poly_bin(FILE *file, mpz_upoly_t pol){
 
   pol->coeffs = malloc(sizeof(mpz_t) * pol->alloc);
   pol->length = pol->alloc;
+
   for(int32_t i = 0; i < pol->length; i++){
     mpz_init(pol->coeffs[i]);
     if(!mpz_inp_raw(pol->coeffs[i], file)){
-      fprintf(stderr, "An error occured when reading file (i=%d)\n", i);
+      fprintf(stderr, "An error occurred when reading file (i=%d)\n", i);
       exit(1);
     }
   }
@@ -1047,7 +1058,7 @@ static inline void get_poly(FILE *file, mpz_upoly_t pol){
   for(int32_t i = 0; i < pol->length; i++){
     mpz_init(pol->coeffs[i]);
     if(!mpz_inp_str(pol->coeffs[i], file, 10)){
-      fprintf(stderr, "An error occured when reading file (i=%d)\n", i);
+      fprintf(stderr, "An error occurred when reading file (i=%d)\n", i);
       exit(1);
     }
   }
@@ -1058,6 +1069,7 @@ static inline void get_single_param_from_file_bin(FILE *file, mpz_param_t param)
   get_poly_bin(file, param->elim);
 
   get_poly_bin(file, param->denom);
+
   if(!fscanf(file, "%ld\n", &param->nvars)){
     fprintf(stderr, "Issue when reading binary file (nvars)\n");
     exit(1);
@@ -1067,8 +1079,17 @@ static inline void get_single_param_from_file_bin(FILE *file, mpz_param_t param)
   param->dquot = param->elim->length - 1;
 
   param->coords = malloc(sizeof(mpz_upoly_t) * param->nvars);
-  for(int32_t i = 0; i < param->nvars; i++){
+  param->cfs = malloc(sizeof(mpz_t) * param->nvars);
+
+  for(int32_t i = 0; i < param->nvars - 1; i++){
     get_poly_bin(file, param->coords[i]);
+
+    mpz_init(param->cfs[i]);
+    if(!mpz_inp_raw(param->cfs[i], file)){
+      fprintf(stderr, "An error occured when reading file (lcm coord i=%d)\n", i);
+      exit(1);
+    }
+
   }
 
 }
@@ -1086,8 +1107,18 @@ static inline void get_single_param_from_file(FILE *file, mpz_param_t param){
   param->dquot = param->elim->length - 1;
 
   param->coords = malloc(sizeof(mpz_upoly_t) * param->nvars);
-  for(int32_t i = 0; i < param->nvars; i++){
+  param->cfs = malloc(sizeof(mpz_t) * param->nvars);
+
+  for(int32_t i = 0; i < param->nvars - 1; i++){
     get_poly(file, param->coords[i]);
+
+    mpz_init(param->cfs[i]);
+
+    if(!mpz_inp_str(param->cfs[i], file, 10)){
+      fprintf(stderr, "An error occured when reading file (i=%d)\n", i);
+      exit(1);
+    }
+
   }
 
 }
