@@ -83,8 +83,8 @@ typedef uint32_t rba_t;  /* reducer binary array */
 typedef uint32_t ind_t;  /* index in hash table structure */
 typedef uint32_t sdm_t;  /* short divmask for faster divisibility checks */
 typedef uint32_t len_t;  /* length type for different structures */
-typedef int16_t exp_t;   /* exponent type */
-typedef int16_t deg_t;   /* (total) degree of polynomial */
+typedef uint16_t exp_t;  /* exponent type */
+typedef int32_t deg_t;   /* (total) degree of polynomial */
 typedef len_t bi_t;      /* basis index of element */
 typedef len_t bl_t;      /* basis load */
 typedef len_t pl_t;      /* pair set load */
@@ -268,10 +268,9 @@ struct ts_t
     len_t *tri;   /* to be reduced rows information in the format */
                   /* basis index1, multiplier1,
                    * basis index2, multiplier2,... */
-    hm_t *lmh;    /* minimal generators of current leading ideal
-                   * presented in the basis hash table */
     len_t lml;    /* number of non-redundant elements in basis */
     deg_t deg;    /* minimal degree to start saturation process */
+    len_t f4rd;   /* round of f4 */
     hm_t *nlms;   /* hashes of new leading monomials represented
                    * in basis hash table */
     len_t rld;    /* load of reducer rows information*/
